import sys

def main(argv:list):
  print('This module does not yet support direct execution. It should be used as a library.')
  print('More information is available at https://github.com/Flippingbinary/wqxlib/wqxlib-python')
  return 0

if __name__ == "__main__":
  main(sys.argv[1:])
